#!/bin/bash

# Copyright 2007 GridWay Team, Distributed Systems Architecture         
# Group, Universidad Complutense de Madrid                                   
#                                                                            
# Licensed under the Apache License, Version 2.0 (the "License"); you may    
# not use this file except in compliance with the License. You may obtain    
# a copy of the License at                                                   
#                                                                            
# http://www.apache.org/licenses/LICENSE-2.0                                 
#                                                                            
# Unless required by applicable law or agreed to in writing, software        
# distributed under the License is distributed on an "AS IS" BASIS,          
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   
# See the License for the specific language governing permissions and        
# limitations under the License.                           

# GLOBUS_CONF
GLOBUS_CLIENT_HOST=`hostname -f`
GLOBUS_CONTAINER_HOST=ursa.dacya.ucm.es

# SAVE ENVIRONMENT, FOR DEVELOPMENT
LOG_FILE=/tmp/sge_starter.$JOB_ID.log
#/usr/bin/env | sort > $LOG_FILE
#echo "Args: $*" >> $LOG_FILE

# GLOBUS ENVIRONMENT
export GLOBUS_LOCATION=/usr/local/globus
. $GLOBUS_LOCATION/etc/globus-user-env.sh

# GO TO JOB DIRECTORY
cd $SGE_O_WORKDIR

S2G_JOB_ARGUMENTS=$@
S2G_BASE_NAME=sge2gt.$JOB_ID
S2G_JOB_SCRIPT_NAME=$S2G_BASE_NAME.sh
S2G_JOB_SCRIPT_FULL=$SGE_O_WORKDIR/$S2G_JOB_SCRIPT_NAME
S2G_JOB_RSL=$SGE_O_WORKDIR/$S2G_BASE_NAME.rsl
S2G_JOB_TAR_NAME=$S2G_BASE_NAME.tarball
S2G_JOB_TAR=$TMP/$S2G_JOB_TAR_NAME
S2G_JOB_TAR_LOCATION=$TMP/$S2G_BASE_NAME
S2G_JOB_STDOUT=$S2G_BASE_NAME.stdout
S2G_JOB_STDERR=$S2G_BASE_NAME.stderr
S2G_WRAPPER_NAME=$S2G_BASE_NAME.wrapper
S2G_WRAPPER=$TMP/$S2G_WRAPPER_NAME


echo -e "#!/bin/bash\n\n\n" > $S2G_JOB_SCRIPT_FULL
cat $JOB_SCRIPT >> $S2G_JOB_SCRIPT_FULL
chmod +x $S2G_JOB_SCRIPT_FULL

# PACK THE WORKING DIRECTORY
tar czf $S2G_JOB_TAR *

# GENERATE WRAPPER
WRAPPER="#!/bin/sh
#cd \${GLOBUS_USER_HOME}
mkdir $S2G_BASE_NAME
cd $S2G_BASE_NAME
tar xzf ../$S2G_JOB_TAR_NAME
./$S2G_JOB_SCRIPT_NAME $S2G_JOB_ARGUMENTS
EXIT_CODE=\$?
rm ../$S2G_JOB_TAR_NAME
cp ../$S2G_JOB_STDOUT .
cp ../$S2G_JOB_STDERR .
tar czf ../$S2G_JOB_TAR_NAME *
exit \$EXIT_CODE
"
echo "$WRAPPER" > $S2G_WRAPPER
chmod +x $S2G_WRAPPER


# GENERATE RSL
RSL="<job>
	<executable>$S2G_WRAPPER_NAME</executable>
	<directory>\${GLOBUS_USER_HOME}</directory>
	<stdout>$S2G_JOB_STDOUT</stdout>
	<stderr>$S2G_JOB_STDERR</stderr>
	<fileStageIn>
		<transfer>
			<sourceUrl>gsiftp://${GLOBUS_CLIENT_HOST}${S2G_JOB_TAR}</sourceUrl>
			<destinationUrl>file:///\${GLOBUS_USER_HOME}/${S2G_JOB_TAR_NAME}</destinationUrl>
		</transfer>
		<transfer>
			<sourceUrl>gsiftp://${GLOBUS_CLIENT_HOST}${S2G_WRAPPER}</sourceUrl>
			<destinationUrl>file:///\${GLOBUS_USER_HOME}/${S2G_WRAPPER_NAME}</destinationUrl>
		</transfer>
	</fileStageIn>
	<fileStageOut>
		<transfer>
			<sourceUrl>file:///\${GLOBUS_USER_HOME}/${S2G_JOB_TAR_NAME}</sourceUrl>
			<destinationUrl>gsiftp://${GLOBUS_CLIENT_HOST}/${SGE_O_WORKDIR}/${S2G_JOB_TAR_NAME}</destinationUrl>
		</transfer>
	</fileStageOut>
</job>"

echo "$RSL" > $S2G_JOB_RSL

# EXECUTE RSL
GLOBUS_STDERR_FILE=$S2G_JOB_SCRIPT_FULL.err
GLOBUS_STDOUT_FILE=$S2G_JOB_SCRIPT_FULL.out
$GLOBUS_LOCATION/bin/globusrun-ws -submit -s -F $GLOBUS_CONTAINER_HOST -f $S2G_JOB_RSL -job-epr-output-file $TMP/epr_file 1>$GLOBUS_STDOUT_FILE 2>$GLOBUS_STDERR_FILE

if [ "x$?" = "x0" ]; then
	# UNPACK TAR
	(
	mkdir $S2G_BASE_NAME
	cd $S2G_BASE_NAME
	tar xzf ../$S2G_JOB_TAR_NAME
	cat $S2G_JOB_STDOUT >> $SGE_STDOUT_PATH
	cat $S2G_JOB_STDERR >> $SGE_STDERR_PATH
	)
	
	# MOVE HELPER FILES TO GLOBUS DIR
	mv $S2G_JOB_SCRIPT_FULL $S2G_BASE_NAME
	mv $S2G_JOB_RSL $S2G_BASE_NAME
	mv $S2G_WRAPPER $S2G_BASE_NAME
	mv $GLOBUS_STDOUT_FILE $S2G_BASE_NAME
	mv $GLOBUS_STDERR_FILE $S2G_BASE_NAME
	mv $S2G_JOB_STDOUT $S2G_BASE_NAME
	mv $S2G_JOB_STDERR $S2G_BASE_NAME
fi

