#!/bin/bash

# Copyright 2007 GridWay Team, Distributed Systems Architecture         
# Group, Universidad Complutense de Madrid                                   
#                                                                            
# Licensed under the Apache License, Version 2.0 (the "License"); you may    
# not use this file except in compliance with the License. You may obtain    
# a copy of the License at                                                   
#                                                                            
# http://www.apache.org/licenses/LICENSE-2.0                                 
#                                                                            
# Unless required by applicable law or agreed to in writing, software        
# distributed under the License is distributed on an "AS IS" BASIS,          
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   
# See the License for the specific language governing permissions and        
# limitations under the License.

# SAVE ENVIRONMENT, FOR DEVELOPMENT
LOG_FILE=/tmp/sge_starter.$JOB_ID.log
#/usr/bin/env > $LOG_FILE

# GLOBUS AND GRIDWAY ENVIRONMENT
export GLOBUS_LOCATION=/usr/local/globus
export GW_LOCATION=/usr/local/gw

. $GLOBUS_LOCATION/etc/globus-user-env.sh
export PATH=$GW_LOCATION/bin:$PATH

# GO TO JOB DIRECTORY
cd $SGE_O_WORKDIR

S2G_JOB_ARGUMENTS=$@
S2G_BASE_NAME=sge2gw.$JOB_ID
S2G_JOB_SCRIPT_NAME=$S2G_BASE_NAME.sh
S2G_JOB_SCRIPT_FULL=$SGE_O_WORKDIR/$S2G_JOB_SCRIPT_NAME
S2G_JOB_TEMPLATE=$SGE_O_WORKDIR/$S2G_BASE_NAME.jt
S2G_JOB_TAR_NAME=$S2G_BASE_NAME.tarball
S2G_JOB_TAR=$TMP/$S2G_JOB_TAR_NAME
S2G_JOB_TAR_LOCATION=$TMP/$S2G_BASE_NAME
S2G_JOB_STDOUT=$S2G_BASE_NAME.stdout
S2G_JOB_STDERR=$S2G_BASE_NAME.stderr
S2G_WRAPPER_NAME=$S2G_BASE_NAME.wrapper
S2G_WRAPPER=$S2G_WRAPPER_NAME


echo -e "#!/bin/bash\n\n\n" > $S2G_JOB_SCRIPT_FULL
cat $JOB_SCRIPT >> $S2G_JOB_SCRIPT_FULL
chmod +x $S2G_JOB_SCRIPT_FULL

# PACK THE WORKING DIRECTORY
tar czf $S2G_JOB_TAR *

# GENERATE WRAPPER
WRAPPER="#!/bin/sh
mkdir $S2G_BASE_NAME
cd $S2G_BASE_NAME
tar xzf ../$S2G_JOB_TAR_NAME
./$S2G_JOB_SCRIPT_NAME $S2G_JOB_ARGUMENTS
EXIT_CODE=\$?
rm ../$S2G_JOB_TAR_NAME
#cp ../$S2G_JOB_STDOUT .
#cp ../$S2G_JOB_STDERR .
tar czf ../$S2G_JOB_TAR_NAME *
exit \$EXIT_CODE
"
echo "$WRAPPER" > $S2G_WRAPPER
chmod +x $S2G_WRAPPER

# GENERATE TEMPLATE
TEMPLATE="
EXECUTABLE=$S2G_WRAPPER_NAME
STDOUT_FILE=$S2G_JOB_STDOUT
STDERR_FILE=$S2G_JOB_STDERR
INPUT_FILES=file://$S2G_JOB_TAR $S2G_JOB_TAR_NAME
OUTPUT_FILES=$S2G_JOB_TAR_NAME
"

echo "$TEMPLATE" > $S2G_JOB_TEMPLATE

# EXECUTE RSL
GW_STDERR_FILE=$S2G_JOB_SCRIPT_FULL.err
GW_STDOUT_FILE=$S2G_JOB_SCRIPT_FULL.out

$GW_LOCATION/bin/gwsubmit -v $S2G_JOB_TEMPLATE 1>$GW_STDOUT_FILE 2>$GW_STDERR_FILE

if [ "x$?" = "x0" ]; then
	GW_JOB_ID=`grep "JOB ID" $GW_STDOUT_FILE | cut -d: -f2`
else
	exit -1
fi

echo -n "$GW_JOB_ID" > $TMP/job_id

$GW_LOCATION/bin/gwwait $GW_JOB_ID

if [ "x$?" = "x0" ]; then
	# UNPACK TAR
	(
	pwd
	ls -l
	mkdir $S2G_BASE_NAME
	cd $S2G_BASE_NAME
	tar xzf ../$S2G_JOB_TAR_NAME
	pwd
	ls -l
	)

	cat $S2G_JOB_STDOUT >> $SGE_STDOUT_PATH
	cat $S2G_JOB_STDERR >> $SGE_STDERR_PATH

	# MOVE HELPER AND LOG FILES TO GLOBUS DIR
	mv $S2G_JOB_SCRIPT_FULL $S2G_BASE_NAME
	mv $S2G_JOB_TEMPLATE $S2G_BASE_NAME
	mv $S2G_WRAPPER $S2G_BASE_NAME
	mv $GW_STDOUT_FILE $S2G_BASE_NAME
	mv $GW_STDERR_FILE $S2G_BASE_NAME
	mv $S2G_JOB_STDOUT $S2G_BASE_NAME
	mv $S2G_JOB_STDERR $S2G_BASE_NAME
fi


